import re

s_dict = None

replace_dict = {'&gt;': '<', '&lt;': '>', '&amp;': '&', '&#246;': 'ö', '&#252;': 'ü', '&auml;': 'ä', '&quot;': '\"',
                    '&#223;': 'ß', '<br>': '\n', '<strong>': '', '</strong>': '', '<em>': '', '</em>': '',
                    '<del>': '[DURCHGESTRICHEN_Anfang]', '</del>': '[DURCHGESTRICHEN_Ende]', '<br />': '\n',
                    '<span class="bbc_u">': '', '</span>': '', '<tt class="bbc_tt">': '', '</tt>': '',
                    '[/color]': '', '<marquee>': '', '</marquee>': '', '<noembed>': '', '</noembed>': '',
                    '<hr>': '\n', '<sup>1</sup>/<sub>2</sub>': '1/2', '<sup>*</sup>': '*', '<sub>1c</sub>': '1c',
                    '<bg></bg>': '', '</li>': '\n', '</ul>': '\n', '<td>': '\t', '</td>': '\t', '</tr><tr>': '\n',
                    '<table class="bbc_table"><tr>': '\n[TABELLE_Anfang]', '</tr></table>': '\n[TABELLE_Ende]\n',
                    '</code>': '[CODE_Ende]', '<div style="text-align: right;">': '', '&nbsp;': ''}

sub_dict = {'(<span style="text-shadow: )(red|yellow|blue)( 1px 1px 1px">)': '',
            '(<span style="color: )(red|Red|blue|Blue|green|Green|navy|Navy|orange|Orange|black|Black|rot|Rot|'
            'purple|Purple|violett|Violett|pink|Pink|yellow|Yellow|beige|Beige|teal|Teal|grey|Grey|#0000BF)(;" '
            'class="bbc_color">)': '',
            '(<span style="font-size: )((\d+\.\d+|\d+)(pt|em)|small)(;" class="bbc_size">)': '',
            '(<span style="font-family: )(Verdana|comic sans ms|arial|georgia|times new roman)'
            '(;" class="bbc_font">)': '',
            '<input type="button" class="spoilerbutton"[^>]*> \(click to show/hide\)': '',
            '<ul class="bbc_list[^>]*">': '\n', '<li(| type="disc")>': '-'}


def smiley_dict(contributions):
    smileys = {}
    for c in contributions:
        smiley_beginnings_list = re.finditer(r'<img src="http://www.forum.diabetesinfo.de/forum/Smileys/',
                                             c.preprocessed_text)
        for smiley in smiley_beginnings_list:
            smiley_begins_at = smiley.span()[0]
            smiley_ends_at = c.preprocessed_text[smiley_begins_at:].find(u'class="smiley">') + smiley_begins_at + 15
            to_be_replaced = c.preprocessed_text[smiley_begins_at:smiley_ends_at]
            smileys[to_be_replaced] = ''
    return smileys


class Contribution:
    def __init__(self, text):
        self.id = self.get_post_id(text)
        self.preprocessed_text = self.inner_class(text)

    def inner_class(self, content):
        inner_begins_at = content.find('<div class="inner"')
        inner_ends_at = content[inner_begins_at:].find('>')
        div_begins_at = content.rfind('</div>')
        return content[inner_ends_at + 1:div_begins_at]

    def get_post_id(self, content):
        return int(re.findall(r'id="msg_[0-9]*">', content)[0][8:-2])

    def fit(self, new_text):
        self.preprocessed_text = new_text

    def smileys(self):
        for key, value in s_dict.items():
            self.fit(self.preprocessed_text.replace(key, value))

    def repl(self):
        for k, v in replace_dict.items():
            self.fit(self.preprocessed_text.replace(k, v))

    def subs(self):
        for k, v in sub_dict.items():
            self.fit(re.sub(k, v, self.preprocessed_text))

    def images(self):
        self.fit(re.sub('<img src=".*(img|img resized)">', '', self.preprocessed_text))

    def links(self):
        link_beginnings = [m.start() for m in re.finditer('<a href=', self.preprocessed_text)]
        for link_beginning in reversed(link_beginnings):
            link_snippet = self.preprocessed_text[link_beginning:]
            link_beginning_ending = link_snippet.find('>')
            link_ending = link_snippet.find('</a>')
            if re.match('.*(Zitat von).*', link_snippet[:link_ending + 4]):
                self.fit(self.preprocessed_text[:link_beginning] + link_snippet[link_ending + 4:])
            else:
                self.fit(self.preprocessed_text[:link_beginning]
                         + '[LINK:' + link_snippet[link_beginning_ending + 1:link_ending] + ']'
                         + link_snippet[link_ending + 4:])

    def quotes(self):
        self.fit(re.sub(r'<div class="topslice_quote">(|Zitat|Zitat von: [^<]*)</div>', '',
                        self.preprocessed_text))
        self.fit(re.sub(r'<div class="botslice_quote"></div>', '', self.preprocessed_text))
        self.fit(re.sub(r'<div class="quoteheader"></div>', '', self.preprocessed_text))
        self.fit(re.sub(r'<div class="quotefooter"></div>', '', self.preprocessed_text))

        quote_beginnings = [m.start() for m in re.finditer('<blockquote class=[^<]*', self.preprocessed_text)]
        for quote_beginning in reversed(quote_beginnings):
            quote_snippet = self.preprocessed_text[quote_beginning:]
            quote_beginning_ending = quote_snippet.find('>')
            quote_ending = quote_snippet.find('</blockquote>')
            self.fit(self.preprocessed_text[:quote_beginning] + '[ZITAT:' +
                     quote_snippet[quote_beginning_ending + 1:quote_ending] + ']' + quote_snippet[quote_ending + 13:])

    def center(self):
        center_beginnings = [m.start() for m in re.finditer('<div align="center">', self.preprocessed_text)]
        for center_beginning in reversed(center_beginnings):
            center_snippet = self.preprocessed_text[center_beginning:]
            center_ending = center_snippet.find('</div>')
            self.fit(self.preprocessed_text[:center_beginning] + center_snippet[20:center_ending]
                     + center_snippet[center_ending + 6:])

    def embed(self):
        embed_beginnings = [m.start() for m in re.finditer('<embed [^>]*>', self.preprocessed_text)]
        for embed_beginning in reversed(embed_beginnings):
            embed_snippet = self.preprocessed_text[embed_beginning:]
            embed_beginning_ending = embed_snippet.find('>')
            embed_ending = embed_snippet.find('</embed>')
            self.fit(self.preprocessed_text[:embed_beginning] + '[APPLICATION:' +
                     embed_snippet[embed_beginning_ending + 1:embed_ending] + ']' + embed_snippet[embed_ending + 8:])

    def spoiler(self):
        self.fit(self.preprocessed_text.replace('<div class="spoilerheader"></div>', ''))
        spoilerbody_beginnings = [m.start() for m in re.finditer('<div class="spoilerbody" style="display: none">',
                                                                 self.preprocessed_text)]
        for spoilerbody_beginning in reversed(spoilerbody_beginnings):
            spoilerbody_snippet = self.preprocessed_text[spoilerbody_beginning:]
            spoilerbody_beginning_ending = spoilerbody_snippet.find('>')
            spoilerbody_ending = spoilerbody_snippet.find('</div>')
            self.fit(self.preprocessed_text[:spoilerbody_beginning] + spoilerbody_snippet
            [spoilerbody_beginning_ending + 1:spoilerbody_ending] + spoilerbody_snippet[spoilerbody_ending + 6:])
        spoiler_beginnings = [m.start() for m in re.finditer('<div class="spoiler">', self.preprocessed_text)]
        for spoiler_beginning in reversed(spoiler_beginnings):
            spoiler_snippet = self.preprocessed_text[spoiler_beginning:]
            spoiler_beginning_ending = spoiler_snippet.find('>')
            spoiler_ending = spoiler_snippet.find('</div>')
            self.fit(self.preprocessed_text[:spoiler_beginning] + spoiler_snippet
            [spoiler_beginning_ending + 1:spoiler_ending] + spoiler_snippet[spoiler_ending + 6:])

    def code(self):
        self.fit(re.sub('<div class="codeheader">Code: [^<]*</div><code class="bbc_code">', '[CODE_Anfang]',
                        self.preprocessed_text))

    def div(self):
        self.fit(self.preprocessed_text.replace('</div>', ''))

    def linebreaks(self):
        self.fit(" ".join(self.preprocessed_text.split("\n")))

    def whitespaces(self):
        self.fit(" ".join(self.preprocessed_text.split()))

    def transform(self):
        self.smileys()
        self.repl()
        self.subs()
        self.images()
        self.links()
        self.quotes()
        self.center()
        self.embed()
        self.spoiler()
        self.code()
        self.div()
        self.linebreaks()
        self.whitespaces()


def transformer(input_data):
    """Returns the input_data in the format needed to merge with annotations.

    :param input_data: a list of raw contribution texts
    :rtype: dictionary with key: msg_id, value: transformed contribution text

    Text specifications after transformation:
    [LINK:link]
    [ZITAT:quote]
    [DURCHGESTRICHEN_Anfang]text[DURCHGESTRICHEN_Ende]
    [APPLICATION:app]
    [TABELLE_Anfang]table[TABELLE_Ende]
    [CODE_Anfang]code[CODE_Ende]
    """
    data_list = [Contribution(e) for e in input_data]
    global s_dict
    s_dict = smiley_dict(data_list)
    for post in data_list:
        post.transform()
    return {d.id: d.preprocessed_text for d in data_list}
